
-- hump Timer module
-- http://vrld.github.com/hump/#timer.lua
require("hump/timer.lua")

learnTimer = Gamestate.new()

function learnTimer:enter(previous)
  menu =
    "This is the timer module example.\n\n" ..
    
    "Escape returns to the menu.\n\n" ..
    
    "Press the corresponding number for demonstrations.\n" ..
    "1. Clear Timer functions.\n" ..
    "2. Clear Squares.\n" ..
    "3. Add a square after 1 second.\n" ..
    "4. Add a square every 1 second for only 10 seconds.\n" ..
    "5. Add a square every 1 second.\n" ..
    "6. Fade in a square.\n" ..
    "7. Add a RAINBOW square!\n" ..
    "8. Epilepsy!"
  
  -- Using squares to demonstrate timer functions
  squares = {}
  function newSquare(update)
    length = math.random(150) + 50
    x = math.random(800-length) + 0.5
    y = math.random(600-length) + 0.5
    red = math.random(255)
    green = math.random(255)
    blue = math.random(255)
    alpha = 255
    local s = {x=x,y=y, length=length, red=red,green=green,blue=blue, alpha=alpha}
    s.update = update or function() end
    return s
  end
  
end

function learnTimer:update(dt)
  for i=#squares, 1, -1 do
    squares[i]:update(dt)
  end
  -- Required for Timer module to function
  Timer.update(dt)
end

function learnTimer.draw()
  for i=#squares, 1, -1 do
    local origColor = {love.graphics.getColor()}
    local square = squares[i]
    love.graphics.setColor(square.red,square.green,square.blue, square.alpha)
    love.graphics.rectangle("line",square.x,square.y,square.length,square.length)
    love.graphics.setColor(unpack(origColor))
  end
  love.graphics.print("#squares: "..#squares.."\n\n"..menu,10,10)
end

function learnTimer:keypressed(key)
  if key=='escape' then
    Gamestate.switch(learnGamestate)
  elseif key=='1' then
    -- Clears all functions in Timer
    Timer.clear()
  elseif key=='2' then
    squares = {}
  elseif key=='3' then
    -- This will call the function after 1 second
    Timer.add(1, function() table.insert(squares,1,newSquare()) end)
  elseif key=='4' then
    -- This will call the passed function every 1 seconds
    Timer.addPeriodic(1, function() table.insert(squares,1,newSquare()) end, 10)
  elseif key=='5' then
    -- This will call the passed function every 1 seconds
    Timer.addPeriodic(1, function() table.insert(squares,1,newSquare()) end)
  elseif key=='6' then
    -- This will create a square that fades in
    local fader = Interpolator(5, function(fraction) return fraction*255 end)
    table.insert(squares,1,newSquare(function(self,dt)
        local valid, alpha = fader(dt)
        if valid then
          self.alpha = alpha end
        end))
  elseif key=='7' then
    -- This will create a square that oscilates colors
    local rainbow = Oscillator(5,
        function(fraction, shift)
          shift = shift or 0
          return math.sin((2*math.pi)*(fraction+shift))*127.5+127.5
        end)
    table.insert(squares,1,newSquare(
        function(self,dt)
          self.red = rainbow(dt,0)
          self.green = rainbow(dt,2/3) 
          self.blue = rainbow(dt,1/3)
        end))
  elseif key=='8' then
    -- This will create a square that oscilates colors
    local epilepsy = Oscillator(5,
        function(fraction,shift)
          shift = shift or 0
          return math.sin((2*math.pi)*(fraction+shift))*127.5+127.5
        end)
    table.insert(squares,1,newSquare(
        function(self,dt)
          self.red = epilepsy(dt)
          self.green = epilepsy(dt+1/3) 
          self.blue = epilepsy(dt+2/3)
        end))
  end
end

function learnTimer:leave()
  menu = nil
  squares = nil
end
